==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/binding/src/lib/ConnectionHandler.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from "@angular/core";
import {BehaviorSubject, Subject} from "rxjs";
import {Deferred} from "./RSUtils";
import {Store} from "@ngrx/store";
import {ConnectionAction} from "./actions/ConnectionAction";
import type {RepetierState, PrinterState, ConnectionState, PrintersState} from "./state/RepetierState";
import {take} from "rxjs/operators";

export class ServerEvent {
  data: any;
  slug: string;
  printer: PrinterState;

  constructor(public event: string) {
  }
}

@Injectable()
export class ConnectionHandler {
  connectionState: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  serverEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  localEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  connectionEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  server: string = window.location.host.replace("4200", "8080");
  httpType = "http";
  private connection: ConnectionState;

  private ws: WebSocket;
  private ignoreErrorsTil = -1;
  // Keep all pending requests here until they get responses
  private callbacks: any = {};
  // Create a unique callback ID to map requests to responses
  private currentCallbackId = 0;
  // Create our websocket object with the address to the websocket
  private wasConnected = false;
  private nextTimeout = 50;
  private openPings = 0;
  private deflist: any = [];
  private disposed = false;

  static isSecure(): boolean {
    return window.location.protocol === "https:";
  }

  constructor(private connectionAction: ConnectionAction,
              private redux: Store<RepetierState>
  ) {
    this.redux.select("connection").subscribe(c => {
      this.connection = c;
    });
    this.ping(); // start pinger
    if (ConnectionHandler.isSecure()) {
      this.httpType = "https";
    }
  }

  get printers(): PrintersState {
    let pe: PrintersState;
    this.redux.select("printers").pipe(take(1)).subscribe(printers => {
      pe = printers;
    });
    return pe;
  }

  fireLocalEvent(event: string, data?: any, printer?: PrinterState) {
    const e: ServerEvent = new ServerEvent(event);
    if (printer) {
      e.printer = printer;
      e.slug = printer.slug;
    } else {
      e.printer = undefined;
      e.slug = undefined;
    }
    e.data = (data ? data : {});
    this.localEvents.next(e);
  }

  connectUsingAPIKey(key: string) {
    this.connectionAction.setConnectionAPI(key, false);
    this.startConnection();
  }

  connectFrontend() {
    this.connectionAction.setConnectionAPI("", true);
    this.startConnection();
  }

  connectUsingLogin(log: string, pw: string) {
    this.connectionAction.setConnectionUser(log, pw);
    this.startConnection();
  }

  getSession(): string {
    if (this.connection) {
      return this.connection.session;
    }
    return '';
  }
  // Do a reconnect or first connect
  startConnection = () => {
    if (this.ws != null && this.ws.readyState === 1) {
      return;
    }
    const auth: any = undefined; // localStorageService.get("session");
    let extra = "";
    this.ignoreErrorsTil = -1; // reset error handling and callbacks

    if (this.connection.frontend) {
      extra += "&front=1";
    }
    if (this.connection.apikey) {
      extra += "&apikey=" + encodeURIComponent((this.connection.apikey));
    } else if (auth && auth.session) {
      extra += "&sess=" + encodeURIComponent(auth.session);
    }
    // console.log('opening socket', (ConnectionHandler.isSecure() ? 'wss://' : 'ws://') + this.server + '/socket/?lang=' + this.redux.getState().connection.language + extra);
    this.ws = new WebSocket((ConnectionHandler.isSecure() ? "wss://" : "ws://") + this.server + "/socket/?lang=" + this.connection.language + extra);
    this.ws.onopen = () => {
      this.connectionAction.connected();
      this.openPings = 0;

      // con.updateImportant();
      const oldCallbacks = this.callbacks;
      this.callbacks = [];
      const tryFlush = () => {
        if (this.ws.readyState !== 1) {
          setTimeout(tryFlush, 20);
          return;
        }
        this.connectionEvents.next(new ServerEvent("updateImportant"));
        // this.updateImportant();
        for (const val of this.deflist) {
          val.resolve(this.ws);
        }
        this.deflist = [];

        this.nextTimeout = 50;
        this.wasConnected = true;
        this.connectionState.next(true);
        this.fireLocalEvent("connected");
      };
      // all commands are unanswered and need to be resend once connection is established
      for (const cbidx in oldCallbacks) {
        if (!oldCallbacks.hasOwnProperty(cbidx)) {
          continue;
        }
        const cb = oldCallbacks[cbidx];
        ((cbFunc) => {
          const d = new Deferred();
          this.deflist.push(d);
          d.promise.then(() => {
            if (!cbFunc.request.ws || cbFunc.request.ws !== this.ws) {
              delete (cbFunc.request.ws);
              this.ws.send(JSON.stringify(cbFunc.request));
              cbFunc.request.ws = this.ws;
            }
          });
        })(cb);
      }
      setTimeout(tryFlush, 10);
    };
    this.ws.onerror = function (/*message*/) {
    };
    this.ws.onclose = (/*message:CloseEvent*/) => {
      if (this.connection.connected) {
        this.connectionAction.disconnected();
        this.connectionState.next(false);
        this.fireLocalEvent("disconnected");
        /*if (!con.firstConnection) {
            con.conDialogVisible = true;
            con.visualAid.onDisconnected();
        }*/
      }
      setTimeout(() => this.startConnection(), this.nextTimeout);
      if (this.nextTimeout < 1000) {
        this.nextTimeout = 2 * this.nextTimeout;
      } else {
        this.nextTimeout = 2000;
      }
    };
    this.ws.onmessage = (message: any) => {
      try {
        this.listener(JSON.parse(message.data));
      } catch (error) {
        console.log("websocket listener");
        console.log(error);
        console.log(message);
      }
    };
  }

  close() {
    this.ws.onclose = function () {
    };
    this.ws.close();
  }

  sendRequest(request): Promise<any> {
    const defer = new Deferred();
    request.callback_id = this.getCallbackId();
    if (request.action === "login") {
      this.ignoreErrorsTil = request.callback_id;
    }
    this.callbacks[request.callback_id] = {
      time: new Date(),
      cb: defer,
      request: request
    };
    if (!this.connection || (this.ws && this.ws.readyState !== 1)) { // buffer commands until connection is open
      const d = new Deferred();
      d.promise.then(() => {
        if (!request.ws || request.ws !== this.ws) {
          delete (request.ws);
          this.ws.send(JSON.stringify(request));
          request.ws = this.ws;
        }
      });
      this.deflist.push(d);
    } else {
      this.ws.send(JSON.stringify(request));
      request.ws = this.ws;
    }
    return defer.promise;
  }

  send(command: string, data?: any, slug?: string): Promise<any> {
    if (!slug) {
      slug = this.printers.active;
    }
    if (!data) {
      data = {};
    }
    return this.sendRequest({action: command, data: data, printer: slug});
  }

  // This creates a new callback ID for a request
  private getCallbackId(): number {
    this.currentCallbackId += 1;
    if (this.currentCallbackId > 10000) {
      this.currentCallbackId = 0;
    }
    return this.currentCallbackId;
  }

  listener(data) {
    const messageObj = data;
    if (typeof (messageObj.session) !== "undefined" && this.connection.session !== messageObj.session) {
      this.connectionAction.setSession(messageObj.session);
    }
    // If an object exists with callback_id in our callbacks object, resolve it
    if (messageObj.callback_id < 0) { // event
      for (const evt of messageObj.data) {
        const e: ServerEvent = new ServerEvent(evt.event);
        e.data = evt.data;
        e.printer = this.printers.printers[evt.printer];
        e.slug = evt.printer;
        this.serverEvents.next(e);
      }
    } else if (this.callbacks[messageObj.callback_id]) {
      const cb = this.callbacks[messageObj.callback_id];
      if (this.ignoreErrorsTil < 0 && this.ignoreErrorsTil > messageObj.callback_id) {
        this.ignoreErrorsTil = -1;
      }
      if (typeof (messageObj.data.permissionDenied) !== "undefined") {
        if (this.ignoreErrorsTil >= 0 && messageObj.callback_id < this.ignoreErrorsTil) {
          cb.cb.reject();
          delete this.callbacks[messageObj.callback_id];
        } else {
          cb.cb.reject();
          delete this.callbacks[messageObj.callback_id];
          this.forceLogin();
        }
        return;
      } else if (typeof (messageObj.data.alert) !== "undefined") {
        cb.cb.reject();
        delete this.callbacks[messageObj.callback_id];
        return;
      }
      cb.cb.resolve(messageObj.data);
      delete this.callbacks[messageObj.callback_id];
    }
  }

  forceLogin() {
    /*    if ($location.path() != '/login')
     $rootScope.pathAfterLogin = $location.path();
     else
     $rootScope.pathAfterLogin = '/';
     if (User.autologin(Service)) return;
     $rootScope.$apply(function () {
     $location.path('/login');
     });*/
  }

  extendPing(timeout: number): void {
    this.send("extendPing", {timeout: timeout});
  }

  ping = () => {
    if (this.connection.connected) {
      if (this.openPings > 10) {
        this.ws.close();
      } else {
        this.openPings++;
        this.send("ping", {"source": "Touchscreen"}).then(() => {
          this.openPings--;
        }); // we are still alive!
      }
    }
    if (!this.disposed) {
      setTimeout(() => this.ping(), 2000);
    }
  }

  networkInterfaces() {
    return this.send("networkInterfaces");
  }
}

==> RSTouchscreenNG-1.4.1.zip: RSTouchscreenNG/projects/binding/src/lib/ConnectionHandler.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from "@angular/core";
import {BehaviorSubject, Subject} from "rxjs";
import {Deferred} from "./RSUtils";
import {Store} from "@ngrx/store";
import {ConnectionAction} from "./actions/ConnectionAction";
import type {RepetierState, PrinterState, ConnectionState, PrintersState} from "./state/RepetierState";
import {take} from "rxjs/operators";

export class ServerEvent {
  data: any;
  slug: string;
  printer: PrinterState;

  constructor(public event: string) {
  }
}

@Injectable()
export class ConnectionHandler {
  connectionState: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  serverEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  localEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  connectionEvents: Subject<ServerEvent> = new Subject<ServerEvent>();
  server: string = window.location.host.replace("4200", "8080");
  httpType = "http";
  private connection: ConnectionState;

  private ws: WebSocket;
  private ignoreErrorsTil = -1;
  // Keep all pending requests here until they get responses
  private callbacks: any = {};
  // Create a unique callback ID to map requests to responses
  private currentCallbackId = 0;
  // Create our websocket object with the address to the websocket
  private wasConnected = false;
  private nextTimeout = 50;
  private openPings = 0;
  private deflist: any = [];
  private disposed = false;

  static isSecure(): boolean {
    return window.location.protocol === "https:";
  }

  constructor(private connectionAction: ConnectionAction,
              private redux: Store<RepetierState>
  ) {
    this.redux.select("connection").subscribe(c => {
      this.connection = c;
    });
    this.ping(); // start pinger
    if (ConnectionHandler.isSecure()) {
      this.httpType = "https";
    }
  }

  get printers(): PrintersState {
    let pe: PrintersState;
    this.redux.select("printers").pipe(take(1)).subscribe(printers => {
      pe = printers;
    });
    return pe;
  }

  fireLocalEvent(event: string, data?: any, printer?: PrinterState) {
    const e: ServerEvent = new ServerEvent(event);
    if (printer) {
      e.printer = printer;
      e.slug = printer.slug;
    } else {
      e.printer = undefined;
      e.slug = undefined;
    }
    e.data = (data ? data : {});
    this.localEvents.next(e);
  }

  connectUsingAPIKey(key: string) {
    this.connectionAction.setConnectionAPI(key, false);
    this.startConnection();
  }

  connectFrontend() {
    this.connectionAction.setConnectionAPI("", true);
    this.startConnection();
  }

  connectUsingLogin(log: string, pw: string) {
    this.connectionAction.setConnectionUser(log, pw);
    this.startConnection();
  }

  getSession(): string {
    if (this.connection) {
      return this.connection.session;
    }
    return '';
  }
  // Do a reconnect or first connect
  startConnection = () => {
    if (this.ws != null && this.ws.readyState === 1) {
      return;
    }
    const auth: any = undefined; // localStorageService.get("session");
    let extra = "";
    this.ignoreErrorsTil = -1; // reset error handling and callbacks

    if (this.connection.frontend) {
      extra += "&front=1";
    }
    if (this.connection.apikey) {
      extra += "&apikey=" + encodeURIComponent((this.connection.apikey));
    } else if (auth && auth.session) {
      extra += "&sess=" + encodeURIComponent(auth.session);
    }
    // console.log('opening socket', (ConnectionHandler.isSecure() ? 'wss://' : 'ws://') + this.server + '/socket/?lang=' + this.redux.getState().connection.language + extra);
    this.ws = new WebSocket((ConnectionHandler.isSecure() ? "wss://" : "ws://") + this.server + "/socket/?lang=" + this.connection.language + extra);
    this.ws.onopen = () => {
      this.connectionAction.connected();
      this.openPings = 0;

      // con.updateImportant();
      const oldCallbacks = this.callbacks;
      this.callbacks = [];
      const tryFlush = () => {
        if (this.ws.readyState !== 1) {
          setTimeout(tryFlush, 20);
          return;
        }
        this.connectionEvents.next(new ServerEvent("updateImportant"));
        // this.updateImportant();
        for (const val of this.deflist) {
          val.resolve(this.ws);
        }
        this.deflist = [];

        this.nextTimeout = 50;
        this.wasConnected = true;
        this.connectionState.next(true);
        this.fireLocalEvent("connected");
      };
      // all commands are unanswered and need to be resent once connection is established
      for (const cbIndex in oldCallbacks) {
        if (!oldCallbacks.hasOwnProperty(cbIndex)) {
          continue;
        }
        const cb = oldCallbacks[cbIndex];
        ((cbFunc) => {
          const d = new Deferred();
          this.deflist.push(d);
          d.promise.then(() => {
            if (!cbFunc.request.ws || cbFunc.request.ws !== this.ws) {
              delete (cbFunc.request.ws);
              this.ws.send(JSON.stringify(cbFunc.request));
              cbFunc.request.ws = this.ws;
            }
          });
        })(cb);
      }
      setTimeout(tryFlush, 10);
    };
    this.ws.onerror = function (/*message*/) {
    };
    this.ws.onclose = (/*message:CloseEvent*/) => {
      if (this.connection.connected) {
        this.connectionAction.disconnected();
        this.connectionState.next(false);
        this.fireLocalEvent("disconnected");
        /*if (!con.firstConnection) {
            con.conDialogVisible = true;
            con.visualAid.onDisconnected();
        }*/
      }
      setTimeout(() => this.startConnection(), this.nextTimeout);
      if (this.nextTimeout < 1000) {
        this.nextTimeout = 2 * this.nextTimeout;
      } else {
        this.nextTimeout = 2000;
      }
    };
    this.ws.onmessage = (message: any) => {
      try {
        this.listener(JSON.parse(message.data));
      } catch (error) {
        console.log("websocket listener");
        console.log(error);
        console.log(message);
      }
    };
  }

  close() {
    this.ws.onclose = function () {
    };
    this.ws.close();
  }

  sendRequest(request): Promise<any> {
    const defer = new Deferred();
    request.callback_id = this.getCallbackId();
    if (request.action === "login") {
      this.ignoreErrorsTil = request.callback_id;
    }
    this.callbacks[request.callback_id] = {
      time: new Date(),
      cb: defer,
      request: request
    };
    if (!this.connection || (this.ws && this.ws.readyState !== 1)) { // buffer commands until connection is open
      const d = new Deferred();
      d.promise.then(() => {
        if (!request.ws || request.ws !== this.ws) {
          delete (request.ws);
          this.ws.send(JSON.stringify(request));
          request.ws = this.ws;
        }
      });
      this.deflist.push(d);
    } else {
      this.ws.send(JSON.stringify(request));
      request.ws = this.ws;
    }
    return defer.promise;
  }

  send(command: string, data?: any, slug?: string): Promise<any> {
    if (!slug) {
      slug = this.printers.active;
    }
    if (!data) {
      data = {};
    }
    return this.sendRequest({action: command, data: data, printer: slug});
  }

  // This creates a new callback ID for a request
  private getCallbackId(): number {
    this.currentCallbackId += 1;
    if (this.currentCallbackId > 10000) {
      this.currentCallbackId = 0;
    }
    return this.currentCallbackId;
  }

  listener(data) {
    const messageObj = data;
    if (typeof (messageObj.session) !== "undefined" && this.connection.session !== messageObj.session) {
      this.connectionAction.setSession(messageObj.session);
    }
    // If an object exists with callback_id in our callbacks object, resolve it
    if (messageObj.callback_id < 0) { // event
      for (const evt of messageObj.data) {
        const e: ServerEvent = new ServerEvent(evt.event);
        e.data = evt.data;
        e.printer = this.printers.printers[evt.printer];
        e.slug = evt.printer;
        this.serverEvents.next(e);
      }
    } else if (this.callbacks[messageObj.callback_id]) {
      const cb = this.callbacks[messageObj.callback_id];
      if (this.ignoreErrorsTil < 0 && this.ignoreErrorsTil > messageObj.callback_id) {
        this.ignoreErrorsTil = -1;
      }
      if (typeof (messageObj.data.permissionDenied) !== "undefined") {
        if (this.ignoreErrorsTil >= 0 && messageObj.callback_id < this.ignoreErrorsTil) {
          cb.cb.reject();
          delete this.callbacks[messageObj.callback_id];
        } else {
          cb.cb.reject();
          delete this.callbacks[messageObj.callback_id];
          this.forceLogin();
        }
        return;
      } else if (typeof (messageObj.data.alert) !== "undefined") {
        cb.cb.reject();
        delete this.callbacks[messageObj.callback_id];
        return;
      }
      cb.cb.resolve(messageObj.data);
      delete this.callbacks[messageObj.callback_id];
    }
  }

  forceLogin() {
    /*    if ($location.path() != '/login')
     $rootScope.pathAfterLogin = $location.path();
     else
     $rootScope.pathAfterLogin = '/';
     if (User.autologin(Service)) return;
     $rootScope.$apply(function () {
     $location.path('/login');
     });*/
  }

  extendPing(timeout: number): void {
    this.send("extendPing", {timeout: timeout});
  }

  ping = () => {
    if (this.connection.connected) {
      if (this.openPings > 10) {
        this.ws.close();
      } else {
        this.openPings++;
        this.send("ping", {"source": "Touchscreen"}).then(() => {
          this.openPings--;
        }); // we are still alive!
      }
    }
    if (!this.disposed) {
      setTimeout(() => this.ping(), 2000);
    }
  }

  networkInterfaces() {
    return this.send("networkInterfaces");
  }
}
